import java.io.*;
import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;
import javax.swing.*;
import java.util.LinkedList;
import java.util.ArrayList;
import java.util.Scanner;
import java.util.Iterator;

//********************************************************************
//  Image.java      Author: Ryo (Rio) Akasaka
//  
//  Last modified: Tue May 1, 2007
//
//  Description: A class that uses a Graphical User Interface to
//  allow the user to manipulate an image by several methods,
//  including cropping, zooming, scrolling, tiling, darken/lightening
//  grayscale, negative, and removal of red, among others.
//  Also allows for the extraction of an encoded message inside
//  a ppm image.
//********************************************************************

public class Image {
  
  /* An array of SimpleButtons */
  private SimpleButton[] buttons;
  
  /* The original picture and the current picture */
  private Picture originalPicture, picture;
  
  /* This function is called when the program starts and is the
   * only way you can configure the user interface.
   */
  private void createButtons() {

    /* How many buttons do you want? */
    int numButtons = 26;
    
    /* Create an array of SimpleButtons */
    buttons = new SimpleButton[numButtons];
    
    int counter = 0;          

    /* 1. Restore the image */    
    buttons[counter++] =      
      new SimpleButton("Restore", new RestoreImage(), Color.green);

    /* 2. Quit the program */    
    buttons[counter++] =      
      new SimpleButton("Quit", new Quit(), Color.red);

    /* 3. Remove all red */    
    buttons[counter++] = 
      new SimpleButton("Remove Red", new RemoveRed());
      
    /* 4. Crop the image */    
    buttons[counter++] = 
      new SimpleButton("Crop ", new Crop());
    
    /* 5. Make negative */
    buttons[counter++] = 
      new SimpleButton("Negative", new Negative());
    
    /* 6. Convert to Grayscale */
    buttons[counter++] =
      new SimpleButton("Grayscale", new Grayscale());
    
    /* 7. Flip Horizontally */
    buttons[counter++] =
      new SimpleButton("Flip Horizontal", new FlipHorizontal());
    
    /* 8. Flip Vertically */
    buttons[counter++] =
      new SimpleButton("Flip Vertical", new FlipVertical());
    
    /* 9. Flip Corners */
    buttons[counter++] =
      new SimpleButton("Flip Corners", new FlipCorners());
    
    /* 10. Lighten the image */
    buttons[counter++] =
      new SimpleButton("Lighten", new Lighten());
    
    /* 11. Darken the image */
    buttons[counter++] =
      new SimpleButton("Darken", new Darken());
    
    /* 12. Polarize */
    buttons[counter++] =
      new SimpleButton("Polarize", new Polarize());
    
    /* 13. Zoom Upper Left */
    buttons[counter++] =
      new SimpleButton("Zoom upper left", new ZoomUpperLeft());
    
    /* 14. Zoom Lower Left */  
    buttons[counter++] =
    new SimpleButton("Zoom lower left", new ZoomLowerLeft());
  
    /* 15. Zoom Upper Right */ 
    buttons[counter++] =
      new SimpleButton("Zoom upper right", new ZoomUpperRight());
    
    /* 16. Zoom Lower Right */   
    buttons[counter++] =
      new SimpleButton("Zoom lower right", new ZoomLowerRight());
    
    /* 17. Zoom Center */
    buttons[counter++] =
       new SimpleButton("Zoom center", new ZoomCenter());
    
    /* EC. Enlarge */
    buttons[counter++] =
      new SimpleButton("Enlarge", new Enlarge());
    
    /* 18. Blur image */
    buttons[counter++] =
      new SimpleButton("Blur image", new Blur());
    
    /* 19. Rotate by 90 degrees */
    buttons[counter++] =
      new SimpleButton("Rotate by 90", new Rotate());
    
    /* 20. Read PPM */    
    buttons[counter++] = 
      new SimpleButton("Read PPM file", new ReadPPMListener());
    
    /* 21. Retrieve message */    
    buttons[counter++] = 
      new SimpleButton("Retrieve Message", new RetrieveListener());
    
    /* EC. Scroll Horizontally */    
    buttons[counter++] = 
      new SimpleButton("Scroll Horizontal", new ScrollHorizontal());
    
    /* EC. Scroll Vertically */    
    buttons[counter++] = 
      new SimpleButton("Scroll Vertical", new ScrollVertical());
    
    /* 22. Tiling effect */    
    buttons[counter++] =
      new SimpleButton("Tile", new Tile()); 

    /* EC. Eight Squares */    
    buttons[counter++] = 
      new SimpleButton("EightSquares", new EightSquares());
    
  }
  
  /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
  /* Action Listeners defined below				             */
  /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

  class RestoreImage implements ActionListener {
    public void actionPerformed (ActionEvent e) {
      picture = new Picture(originalPicture);
      pictureFrame.refresh(picture);
    }
  }
  
  class Quit implements ActionListener {
    public void actionPerformed (ActionEvent e) {
      System.exit(0);
    }
  }
  
  class Rotate implements ActionListener {
    public void actionPerformed (ActionEvent e) {

      /* Query the current width and height of the image */      
      int imageWidth = picture.getWidth();
      int imageHeight = picture.getHeight();
      Picture replace = new Picture(picture);
      
      /* Resize the frame */
      Picture copy = new Picture(imageHeight, imageWidth);
      for (int x = 0; x < imageWidth; x++) { 
	for (int y = 0; y < imageHeight; y++) { 
	  Pixel pix = replace.getPixel(x,y);
	  copy.setPixel(imageHeight - y - 1, x, pix);
     }
      }
      picture = copy;
      pictureFrame.refresh(picture); }
  }
  
  class RemoveRed implements ActionListener {
    public void actionPerformed (ActionEvent e) {
      
      /* Query the current width and height of the image */
      int imageWidth = picture.getWidth();
      int imageHeight = picture.getHeight();
      
      /* Remove the red from the image */
      for (int x = 0; x < imageWidth; x++) { 
	for (int y = 0; y < imageHeight; y++) { 
	  Pixel pix = picture.getPixel(x,y); 
	  pix.setRed(0);
	  picture.setPixel(x, y, pix); 
	} 
      }   
      pictureFrame.refresh(picture);
    }
  }
  
  class Tile implements ActionListener {
    public void actionPerformed (ActionEvent e) {
      
      /* Query the current width and height of the image */
      int imageWidth = picture.getWidth();
      int imageHeight = picture.getHeight();
      Picture replace = new Picture(picture);
      /* Select quadrant */
      int countx = -2;
      for (int x = 0; x <= imageWidth/2; x++) {
	int county = -2;
	if (countx + 2 < imageWidth) countx += 2;
	else if (countx + 1 < imageWidth) countx += 1;
      	
       	for (int y = 0; y <= imageHeight/2; y++) {
	  if (county + 2 < imageHeight) county += 2;
	  else if (county + 1 < imageHeight) county += 1;
	  Pixel pix = replace.getPixel(countx, county);
	  picture.setPixel(x, y, pix);
	}
      }
      for (int x = 0; x < imageWidth/2; x++) {
	for (int y = 0; y < imageHeight/2; y++) {
	  Pixel pix = picture.getPixel(x, y);
	  picture.setPixel(imageWidth - x, y, pix);
	}
      }
     for (int x = 0; x < imageWidth; x++) {
       for (int y = 0; y < imageHeight/2; y++) {
	 Pixel pix = picture.getPixel(x, y);
	 picture.setPixel(x, imageHeight - y - 1, pix);
       }
     } 
     pictureFrame.refresh(picture);
    }
  }
  
  class Blur implements ActionListener {
    public void actionPerformed (ActionEvent e) {
      
      /* Query the current width and height of the image */
      int imageWidth = picture.getWidth();
      int imageHeight = picture.getHeight();
      Picture replace = new Picture(picture);
      
      for (int x = 0; x < imageWidth; x++) {
	for (int y = 0; y < imageHeight; y++) {
	  
	  int red = 0;
	  int green = 0;
	  int blue = 0;
	  int count = 0;
	  int startx = -1;
	  int starty = -1;
	  int endx = 1;
	  int endy = 1;
      	  
	  /* Along left edge */
	  if ((x - 1) <= 0) { startx = 0; }
	  /* Along top edge */
	  if ((y - 1) <= 0) { starty = 0; }
	  /* Along bottom edge */
	  if ((y + 1) >= imageHeight) { endy = 0; }
	  /* Along right edge */
	  if ((x + 1) >= imageWidth) { endx = 0; }
	  
	  /* Standard 3 x 3 square */
	  for (int valx = startx; valx <= endx; valx++) {
	    for (int valy = starty; valy <= endy; valy++) {
	      count++;
	      Pixel pix = replace.getPixel(x + valx, y + valy);
	      red += pix.getRed();
	      green += pix.getGreen();
	      blue += pix.getBlue();
	    }
	  }
	  Pixel pix = new Pixel(0,0,0);
	  pix.setRed(red/count);
	  pix.setGreen(green/count);
	  pix.setBlue(blue/count);
	  picture.setPixel(x, y, pix);
	}
      }
      pictureFrame.refresh(picture);
    }
  }
  
  class ZoomUpperLeft implements ActionListener {
    public void actionPerformed (ActionEvent e) {
      
      /* Query the current width and height of the image */
      int imageWidth = picture.getWidth();
      int imageHeight = picture.getHeight();
      Picture replace = new Picture(picture);

      int xcount = -2;
      for (int x = 0; x < imageWidth/2; x++) {
	int ycount = -2;
	if (xcount + 2 < imageWidth) xcount += 2;
	else if (xcount + 1 < imageWidth) xcount += 1;
	for (int y = 0; y < imageHeight/2; y++) {
	  if (ycount + 2 < imageHeight) ycount += 2;
	  else if (ycount + 1 < imageWidth) ycount += 1;
	  Pixel pix = replace.getPixel(x,y); 
	  picture.setPixel(xcount, ycount, pix);
	  picture.setPixel(xcount + 1, ycount, pix);
	  picture.setPixel(xcount, ycount + 1, pix);
	  picture.setPixel(xcount + 1, ycount + 1, pix);
	}
      }
      pictureFrame.refresh(picture);
    }
  }
  
  
  class ZoomLowerRight implements ActionListener {
    public void actionPerformed (ActionEvent e) {
      
      /* Query the current width and height of the image */
      int imageWidth = picture.getWidth();
      int imageHeight = picture.getHeight();
      Picture replace = new Picture(picture);

      int xcount = -2;
      for (int x = (imageWidth/2); x < imageWidth; x++) {
	int ycount = -2;
	if (xcount + 2 < imageWidth) xcount += 2;
	else if (xcount + 1 < imageWidth) xcount += 1;
	for (int y = (imageHeight/2); y < imageHeight; y++) {
	  if (ycount + 2 < imageHeight) ycount += 2;
	  else if (ycount + 1 < imageWidth) ycount += 1;
	  Pixel pix = replace.getPixel(x,y); 
	  picture.setPixel(xcount, ycount, pix);
	  picture.setPixel(xcount + 1, ycount, pix);
	  picture.setPixel(xcount, ycount + 1, pix);
	  picture.setPixel(xcount + 1, ycount + 1, pix);
	}
      }
      pictureFrame.refresh(picture);
    }
  }
  
  class ZoomLowerLeft implements ActionListener {
    public void actionPerformed (ActionEvent e) {
      
      /* Query the current width and height of the image */
      int imageWidth = picture.getWidth();
      int imageHeight = picture.getHeight();
      Picture replace = new Picture(picture);

      int xcount = -2;
      for (int x = 0; x < (imageWidth/2); x++) {
	int ycount = -2;
	if (xcount + 2 < imageWidth) xcount += 2;
	else if (xcount + 1 < imageWidth) xcount += 1;
	for (int y = (imageHeight/2); y < imageHeight; y++) {
	  if (ycount + 2 < imageHeight) ycount += 2;
	  else if (ycount + 1 < imageWidth) ycount += 1;
	  Pixel pix = replace.getPixel(x,y);
	  picture.setPixel(xcount, ycount, pix);
	  picture.setPixel(xcount + 1, ycount, pix);
	  picture.setPixel(xcount, ycount + 1, pix);
	  picture.setPixel(xcount + 1, ycount + 1, pix);
	}
      }
      pictureFrame.refresh(picture);
    }
  }
  
  class ZoomUpperRight implements ActionListener {
    public void actionPerformed (ActionEvent e) {
      
      /* Query the current width and height of the image */
      int imageWidth = picture.getWidth();
      int imageHeight = picture.getHeight();
      
      Picture replace = new Picture(picture);
     
      int xcount = -2;
      for (int x = (imageWidth/2); x < imageWidth; x++) {
	int ycount = -2;
	if (xcount + 2 < imageWidth) xcount += 2;
	else if (xcount + 1 < imageWidth) xcount += 1;
	for (int y = 0; y < (imageHeight/2); y++) {
	  if (ycount + 2 < imageHeight) ycount += 2;
	  else if (ycount + 1 < imageWidth) ycount += 1;
	  Pixel pix = replace.getPixel(x,y);
	  picture.setPixel(xcount, ycount, pix);
	  picture.setPixel(xcount + 1, ycount, pix);
	  picture.setPixel(xcount, ycount + 1, pix);
	  picture.setPixel(xcount + 1, ycount + 1, pix);
	}
      }
      pictureFrame.refresh(picture);
    }
  }


  class ZoomCenter implements ActionListener {
    public void actionPerformed (ActionEvent e) {
      
      /* Query the current width and height of the image */
      int imageWidth = picture.getWidth();
      int imageHeight = picture.getHeight();
      Picture replace = new Picture(picture);
      /* Select quadrant */
      int xcount = -2;
      for (int x = (imageWidth/4); x < 3*imageWidth/4; x++) {
	int ycount = -2;
	if (xcount + 2 < imageWidth) xcount += 2;
	for (int y = (imageHeight/4); y < 3*imageHeight/4; y++) {
	  if (ycount + 2 < imageHeight) ycount += 2;
	  Pixel pix = replace.getPixel(x,y); 
	  picture.setPixel(xcount, ycount, pix);
	  picture.setPixel(xcount + 1, ycount, pix);
	  picture.setPixel(xcount, ycount + 1, pix);
	  picture.setPixel(xcount + 1, ycount + 1, pix);
	}
      }
      pictureFrame.refresh(picture);
    }
  }
  
  class Enlarge implements ActionListener {
    public void actionPerformed (ActionEvent e) {
      
      /* Query the current width and height of the image */
      int imageWidth = picture.getWidth();
      int imageHeight = picture.getHeight();
      Picture replace = new Picture(picture);
      
      /* Resize the picture frame */
      Picture copy = new Picture(imageWidth*2, imageHeight*2);

      int xcount = -2;
      for (int x = 0; x < imageWidth; x++) {
	int ycount = -2;
	if (xcount + 2 < imageWidth*2) xcount += 2;
	else if (xcount + 1 < imageWidth*2) xcount += 1;
	for (int y = 0; y < imageHeight; y++) {
	  if (ycount + 2 < imageHeight*2) ycount += 2;
	  else if (ycount + 1 < imageWidth*2) ycount += 1;
	  Pixel pix = replace.getPixel(x,y); 
	  copy.setPixel(xcount, ycount, pix);
	  copy.setPixel(xcount + 1, ycount, pix);
	  copy.setPixel(xcount, ycount + 1, pix);
	  copy.setPixel(xcount + 1, ycount + 1, pix);
	}
      }
      picture = copy;
      pictureFrame.refresh(picture);
    }
  }

  class Crop implements ActionListener {
    public void actionPerformed (ActionEvent e) {
      
      int imageWidth = picture.getWidth();
      int imageHeight = picture.getHeight();
      System.out.println("The picture has dimensions " + imageWidth + " x " + imageHeight + " pixels.");
      Scanner filein;
      String filename;
      Scanner keyin = new Scanner(System.in);
      
      System.out.print("Please enter top left (x y): ");
      int topx = keyin.nextInt();
      int topy = keyin.nextInt();
      
      System.out.print("Please enter bottom right (x y): ");
      int botx = keyin.nextInt();
      int boty = keyin.nextInt();
      Picture copy = new Picture(botx-topx, boty-topy);
      
      for (int x = topx; x < botx; x++) {
	for (int y = topy; y < boty; y++) {
	  Pixel pix = picture.getPixel(x,y);
	  copy.setPixel(x - topx, y - topy, pix);
	}
      }
      picture = copy;
      pictureFrame.refresh(picture);
    }
  }
  
  
  class Negative implements ActionListener {
    public void actionPerformed (ActionEvent e) {
      
      int imageWidth = picture.getWidth();
      int imageHeight = picture.getHeight();
      
      for (int x = 0; x < imageWidth; x++) { 
	for (int y = 0; y < imageHeight; y++) { 
	  Pixel pix = picture.getPixel(x,y);
	  pix.setRed(255 - pix.getRed());
	  pix.setBlue(255 - pix.getBlue());
	  pix.setGreen(255 - pix.getGreen());
	  picture.setPixel(x, y, pix);
	}
      }
      pictureFrame.refresh(picture); }
  }
  
  class Darken implements ActionListener {
    public void actionPerformed (ActionEvent e) {

      int imageWidth = picture.getWidth();
      int imageHeight = picture.getHeight();
      
      for (int x = 0; x < imageWidth; x++) { 
	for (int y = 0; y < imageHeight; y++) { 
	  Pixel pix = picture.getPixel(x,y);
	  if (pix.getRed() > 10)
	    pix.setRed(pix.getRed() - 10);
	  if (pix.getBlue() > 10)
	    pix.setBlue(pix.getBlue() - 10);
	  if (pix.getGreen() > 10)
	    pix.setGreen(pix.getGreen() - 10);
	  picture.setPixel(x, y, pix);
	}
      }
      pictureFrame.refresh(picture); }
  }
  
  
  class Lighten implements ActionListener {
    public void actionPerformed (ActionEvent e) {
      
      int imageWidth = picture.getWidth();
      int imageHeight = picture.getHeight();
      
      for (int x = 0; x < imageWidth; x++) { 
	for (int y = 0; y < imageHeight; y++) { 
	  Pixel pix = picture.getPixel(x,y);
	  if (pix.getRed() < 245)
	    pix.setRed(pix.getRed() + 10);
	  if (pix.getBlue() < 245)
	    pix.setBlue(pix.getBlue() + 10);
	  if (pix.getGreen() < 245)
	    pix.setGreen(pix.getGreen() + 10);
	  picture.setPixel(x, y, pix);
	}
      }
       pictureFrame.refresh(picture); }
  }
  
  class FlipHorizontal implements ActionListener {
    public void actionPerformed (ActionEvent e) {
      
      int imageWidth = picture.getWidth();
      int imageHeight = picture.getHeight();
         
      for (int x = 0; x < imageWidth/2; x++) { 
	for (int y = 0; y < imageHeight; y++) { 
	  Pixel pix = picture.getPixel(x,y);
	  Pixel temp = picture.getPixel((imageWidth - x - 1), y);
	  picture.setPixel(x, y, temp);
	  picture.setPixel(imageWidth - x - 1, y, pix);
	}
      }
      pictureFrame.refresh(picture); }
  }
  
  class FlipVertical implements ActionListener {
    public void actionPerformed (ActionEvent e) {
      
      int imageWidth = picture.getWidth();
      int imageHeight = picture.getHeight();
      
      for (int x = 0; x < imageWidth/2; x++) { 
	for (int y = 0; y < imageHeight; y++) { 
	  Pixel pix = picture.getPixel(x,y);
	  Pixel temp = picture.getPixel((imageWidth - x - 1), (imageHeight - y - 1));
	  picture.setPixel(x, y, temp);
	  picture.setPixel(imageWidth - x - 1, imageHeight - y -1, pix);
	}
      }
      pictureFrame.refresh(picture); }
  }
  
  
  class FlipCorners implements ActionListener {
    public void actionPerformed (ActionEvent e) {
      
      /* Query the current width and height of the image */
      int imageWidth = picture.getWidth();
      int imageHeight = picture.getHeight();
      Picture replace = new Picture(picture);

      for (int x = 0; x < imageWidth; x++) {
	for (int y = 0; y < imageHeight; y++) {
	  Pixel pix = replace.getPixel(x,y); 
	  picture.setPixel(imageWidth - x - 1, imageHeight - y - 1, pix);
	}
      }
      pictureFrame.refresh(picture);
    }
  }
  
  class Grayscale implements ActionListener {
    public void actionPerformed (ActionEvent e) {
      
      int imageWidth = picture.getWidth();
      int imageHeight = picture.getHeight();
  
      for (int x = 0; x < imageWidth; x++) { 
	for (int y = 0; y < imageHeight; y++) { 
	  Pixel pix = picture.getPixel(x,y);
	  int average = ((pix.getRed() + pix.getBlue() + pix.getGreen())/3);
	  pix.setRed(average);
	  pix.setBlue(average);
	  pix.setGreen(average);
	  picture.setPixel(x, y, pix);
	}
      }
      pictureFrame.refresh(picture); 
    }
  }   
 
  
  class Polarize implements ActionListener {
    public void actionPerformed (ActionEvent e) {
      
      int imageWidth = picture.getWidth();
      int imageHeight = picture.getHeight();
           
      int red = 0, blue = 0, green = 0;
      for (int x = 0; x < imageWidth; x++) { 
	for (int y = 0; y < imageHeight; y++) { 
	  Pixel pix = picture.getPixel(x,y);
	  red += pix.getRed();
	  blue += pix.getBlue();
	  green += pix.getGreen();
	}
      }
      int total = imageWidth * imageHeight;
      red = red/total;
      blue = blue/total;
      green = green/total;
      for (int x = 0; x < imageWidth; x++) { 
	for (int y = 0; y < imageHeight; y++) { 
	  Pixel pix = picture.getPixel(x,y);
	  if (pix.getRed() < red) pix.setRed(0);
	  else pix.setRed(255);
	  if (pix.getBlue() < blue) pix.setBlue(0);
	  else pix.setBlue(255);
	  if (pix.getGreen() < green) pix.setGreen(0);
	  else pix.setGreen(255);
	  picture.setPixel(x, y, pix);
	}
      }
      pictureFrame.refresh(picture); 
    }
  }   
  
  class ReadPPMListener implements ActionListener
  {
    public void actionPerformed(ActionEvent e) 
    {
      System.out.println("Input PPM filename: ");
      Scanner sc = new Scanner(System.in);
      String filename = sc.nextLine();
    try {
	Scanner filein = new Scanner (new File(filename));
	System.out.println(filename + " opened. ");
	filein.nextLine(); // skip the P3
	int imageWidth = filein.nextInt(); 
	int imageHeight = filein.nextInt();
	picture = new Picture(imageWidth, imageHeight);
	filein.nextLine(); //skip eoln
	filein.nextLine(); // skip the 255

	for (int y = 0; y < imageHeight; y++) {
	  for (int x=0; x<imageWidth;x++)
	    { 
	      Pixel pix = picture.getPixel(x,y);
	      pix.setRed(filein.nextInt());
	      pix.setGreen(filein.nextInt());
	      pix.setBlue(filein.nextInt());
	      picture.setPixel(x,y,pix);
	    }
	}
	filein.close();
	pictureFrame.refresh(picture);
      }
      catch (Exception ex){
	System.out.println("File not found.");
      };
    }
  }
  
  class RetrieveListener implements ActionListener
  {
    public void actionPerformed(ActionEvent e)
    {
      Scanner filein;
      String filename;
      Scanner keyin = new Scanner(System.in);
      System.out.print("Please enter coordinates (x y): ");
      int x = keyin.nextInt();
      int y = keyin.nextInt();
      
      int total = 0;
      int count = 0;
      
      for (int i = 0; i <3; i++) {
	count++;
	Pixel pix = picture.getPixel(x + i, y);
	int redr = ((pix.getRed() % 2));
	int red = ((pix.getRed() % 2) << 9-count);
	count++;
	int green = ((pix.getGreen() % 2) << 9-count);
	int greenr = (pix.getGreen() % 2);
	count++;
	int blue = ((pix.getBlue() % 2) << 9-count);
	int bluer = (pix.getBlue() % 2);
	total = red + green + blue + total;
      }
      int move = 0;
      for (int i = 0; i < total; i++) {
	count = 0;
	int hex = 0;
	int red = 0;
	for (int j = 0; j < 3; j++) {
	  Pixel pix = picture.getPixel(x + 3 + move, y);
	  if (count != 0) { 
	    count++; 
	    red = ((pix.getRed() % 2) << 8-count);
	  }
	  count++;
	  int green = ((pix.getGreen() % 2) << 8-count);
	  count++;
	  int blue = ((pix.getBlue() % 2) << 8-count);
	  hex = red + green + blue + hex;
	  move++;
	}
	System.out.print((char) hex);
      }
      System.out.println();
    }
  }
  
  
  class ScrollHorizontal implements ActionListener
  {
    public void actionPerformed(ActionEvent e) 
    {
      Picture replace = new Picture(picture);
      int imageWidth = picture.getWidth();
      int imageHeight = picture.getHeight();
      System.out.println("Input percentage (0 - 1.0): ");
      Scanner sc = new Scanner(System.in);
      double percentage = sc.nextDouble();
      int line = (int) (imageWidth* percentage);
      for (int x = 0; x < imageWidth; x++) {
	for (int y = 0; y < imageHeight; y++) {
	  Pixel pix = replace.getPixel(x, y);
	  int xcord = x + line;
	  if (x+line >= imageWidth) xcord = ((x+line) % imageWidth);
	  picture.setPixel( xcord, y, pix);
	}
      }
      pictureFrame.refresh(picture); 
    }
  }
  
  
  class ScrollVertical implements ActionListener
  {
    public void actionPerformed(ActionEvent e) 
    {
      Picture replace = new Picture(picture);
      int imageWidth = picture.getWidth();
      int imageHeight = picture.getHeight();
      System.out.println("Input percentage (0 - 1.0): ");
      Scanner sc = new Scanner(System.in);
      double percentage = sc.nextDouble();
      int line = (int) (imageHeight* percentage);
      for (int x = 0; x < imageWidth; x++) {
	for (int y = 0; y < imageHeight; y++) {
	  Pixel pix = replace.getPixel(x, y);
	  int ycord = y + line;
	  if (y+line >= imageHeight) ycord = ((y+line) % imageHeight);
	  picture.setPixel( x, ycord, pix);
	}
      }
      pictureFrame.refresh(picture); 
    }
  }  
  
  /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
  /* Simple hard-coded (inefficient) version of Eight Squares  */
  /* as a demonstration that it can be done.	               */
  /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
    
  class EightSquares implements ActionListener
  {
    public void actionPerformed(ActionEvent e) 
    {
      Picture replace = new Picture(picture);
      int imageWidth = picture.getWidth();
      int imageHeight = picture.getHeight();
      int j = 0;
      int k = 0;

      // Top left
      for (int x = 0; x < imageWidth/3; x++) {
	for (int y = 0; y < imageHeight/3; y++) {
	  Pixel pix = replace.getPixel(x, y);
	  picture.setPixel(x + imageWidth/3, y + imageHeight/3, pix); // Place middle middle
	}
      }
      
      // Middle top            
      for (int x = imageWidth/3; x < imageWidth*2/3; x++) {
	for (int y = 0; y < imageHeight/3; y++) {
	  Pixel pix = replace.getPixel(x, y);
	  picture.setPixel(x + imageWidth/3, y, pix); // Place top right
	}
      }
                        
      // Top right           
      for (int x = imageWidth*2/3; x < imageWidth; x++) {
	for (int y = 0; y < imageHeight/3; y++) {
	  Pixel pix = replace.getPixel(x, y);
	  picture.setPixel(x - imageWidth*2/3, y, pix); // Place top left
	}
      }
      
      // Middle left
      for (int x = 0; x < imageWidth/3; x++) {
	for (int y = imageHeight/3; y < imageHeight*2/3; y++) {
	  Pixel pix = replace.getPixel(x, y);
	  picture.setPixel(x + imageWidth/3, y-imageHeight/3, pix); // Place top middle
	}
      }
      
      // Middle middle
      for (int x = imageWidth/3; x < imageWidth*2/3; x++) {
	for (int y = imageHeight/3; y < imageHeight*2/3; y++) {
	  Pixel pix = replace.getPixel(x, y);
	  picture.setPixel(x - imageWidth/3, y, pix); // Place top middle
	}
      }
      
      // Middle right
      for (int x = imageWidth*2/3; x < imageWidth; x++) {
	for (int y = imageHeight/3; y < imageHeight*2/3; y++) {
	  Pixel pix = replace.getPixel(x, y);
	  picture.setPixel(x - imageWidth/3, y + imageHeight/3, pix); // Place middle bottom
	}
      }
      // Bottom left
      for (int x = 0; x < imageWidth/3; x++) {
	for (int y = imageHeight*2/3; y < imageHeight; y++) {
	  Pixel pix = replace.getPixel(x, y);
	  picture.setPixel(x + imageWidth*2/3, y - imageHeight/3, pix); // Place middle right
	}
      } 
      // Bottom right
      for (int x = imageWidth*2/3; x < imageWidth; x++) {
	for (int y = imageHeight*2/3; y < imageHeight; y++) {
	  Pixel pix = replace.getPixel(x, y);
	  picture.setPixel(x - imageWidth*2/3, y, pix); // Place left bottom
	}
      } 
      
      // Make black
      for (int x = imageWidth*2/3; x < imageWidth; x++) {
	for (int y = imageHeight*2/3; y < imageHeight; y++) {
	  Pixel pix = new Pixel(0,0,0);
	  picture.setPixel(x, y, pix); 
	}
      }     
      pictureFrame.refresh(picture); 
    }
  }
  
  /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
  /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
  /* There is no code which you need to alter below this line. */
  /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
  /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
  
  /* The frame which holds the menu. */
  private JFrame menuFrame;
  
  /* The frame which holds the image. */
  private PictureFrame pictureFrame;
  
  /* The constructor for this class. */
  Image(String filename) {
    setupMenuFrame();
    setupImageFrame(filename);
  }

  /* You should not alter this method. */
  public static void main (String[] args) {
    Image app;
    
    if (args.length == 1) {
      File f = new File (args[0]);
      if (f.exists()) {
	app = new Image(args[0]);
      } else {
	System.err.println(args[0] + ": File not found.");
	System.exit(0);
      }
    } else {
      System.err.println("Image file required.");
      System.exit(0);
    }
  }
  
  
  /* You should not alter this method.  However, it has been commented
   * for those who may be interested in understanding how the GUI is
   * created.    */
  private void setupMenuFrame() {

    /* Create the buttons */
    createButtons();

    /* Create a JPanel to put the buttons on */
    JPanel menuPanel = new JPanel();

    /* Make the menu background black */
    menuPanel.setBackground(Color.black);

    /* Make sure the buttons line up nicely */
    menuPanel.setLayout(new GridLayout(buttons.length/2,2));

    /* Add buttons to the menu, skipping the Restore and Quit buttons */
    for (int i = 0; i < buttons.length; i++) {
      menuPanel.add(buttons[i]);
    }

    /* Create the menu JFrame which will hold the menu JPanel */
    menuFrame = new JFrame("Image Manipulation Menu");

    /* The JPanel is now added to the menu JFrame */
    menuFrame.getContentPane().add(menuPanel, BorderLayout.CENTER);

    /* If the user closes the window, our program should exit */
    menuFrame.addWindowListener(new WindowAdapter() {
	public void windowClosing(WindowEvent e) {
	  System.exit(0);
	}
      });

    /* Optimally size the menuFrame */
    menuFrame.pack();

    /* Place the menuFrame near the top-left of the screen */
    menuFrame.setLocation(20,20);

    /* Make the menuFrame visible */
    menuFrame.setVisible(true);
  }


  /* You should not alter this method.  However, it has been commented
   * for those who may be interested in understanding how the GUI is
   * created.    */
  private void setupImageFrame(String filename) {

    /* Read a color picture from a file */
    picture = new Picture(filename);

    /* Make a copy of the picture in case we need it again */
    originalPicture = new Picture(picture);

    /* Put the picture in a PictureFrame called "Image" */
    pictureFrame = new PictureFrame("Image", picture);

    /* Quit the program if the user closes the PictureFrame */
    pictureFrame.addWindowListener(new WindowAdapter() {
	public void windowClosing(WindowEvent e) {
	  System.exit(0);
	}
      });

    /* Make the PictureFrame the optimal size for this picture */
    pictureFrame.pack();

    /* Place the PictureFrame 25 pixels to right of the MenuFrame */
    Point p = menuFrame.getLocation();
    pictureFrame.setLocation(p.x+menuFrame.getSize().width+25, p.y);

    /* Make the PictureFrame visible */
    pictureFrame.setVisible(true);

  }
}
    
